@@ -22,11 +22,11 @@ import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 
-import org.bouncycastle.asn1.ASN1Encodable;
+import org.bouncycastle.asn1.ASN1Boolean;
+import org.bouncycastle.asn1.ASN1Encoding;
+import org.bouncycastle.asn1.ASN1GeneralizedTime;
+import org.bouncycastle.asn1.ASN1Integer;
 import org.bouncycastle.asn1.ASN1ObjectIdentifier;
-import org.bouncycastle.asn1.DERBoolean;
-import org.bouncycastle.asn1.DERGeneralizedTime;
-import org.bouncycastle.asn1.DERInteger;
 import org.bouncycastle.asn1.DERNull;
 import org.bouncycastle.asn1.DERSet;
 import org.bouncycastle.asn1.cms.Attribute;
@@ -71,14 +71,13 @@ public class TimeStampTokenGenerator
 
     GeneralName tsa = null;
     
-    private String  tsaPolicyOID;
+    private ASN1ObjectIdentifier  tsaPolicyOID;
 
     PrivateKey      key;
     X509Certificate cert;
     String          digestOID;
     AttributeTable  signedAttr;
     AttributeTable  unsignedAttr;
-    CertStore       certsAndCrls;
 
     private List certs = new ArrayList();
     private List crls = new ArrayList();
@@ -94,7 +93,7 @@ public class TimeStampTokenGenerator
         throws IllegalArgumentException, TSPException
     {
         this.signerInfoGen = signerInfoGen;
-        this.tsaPolicyOID = tsaPolicy.getId();
+        this.tsaPolicyOID = tsaPolicy;
 
         if (!signerInfoGen.hasAssociatedCertificate())
         {
@@ -143,6 +142,20 @@ public class TimeStampTokenGenerator
         this(key, cert, digestOID, tsaPolicyOID, null, null);
     }
 
+    /**
+     * basic creation - only the default attributes will be included here.
+     * @deprecated use SignerInfoGenerator constructor
+     */
+    public TimeStampTokenGenerator(
+        PrivateKey      key,
+        X509Certificate cert,
+        ASN1ObjectIdentifier          digestOID,
+        String          tsaPolicyOID)
+        throws IllegalArgumentException, TSPException
+    {
+        this(key, cert, digestOID.getId(), tsaPolicyOID, null, null);
+    }
+
     /**
      * create with a signer with extra signed/unsigned attributes.
      * @deprecated use SignerInfoGenerator constructor
@@ -159,7 +172,7 @@ public class TimeStampTokenGenerator
         this.key = key;
         this.cert = cert;
         this.digestOID = digestOID;
-        this.tsaPolicyOID = tsaPolicyOID;
+        this.tsaPolicyOID = new ASN1ObjectIdentifier(tsaPolicyOID);
         this.unsignedAttr = unsignedAttr;
 
         //
@@ -341,7 +354,7 @@ public class TimeStampTokenGenerator
             throw new IllegalStateException("can only use this method with SignerInfoGenerator constructor");
         }
 
-        ASN1ObjectIdentifier digestAlgOID = new ASN1ObjectIdentifier(request.getMessageImprintAlgOID());
+        ASN1ObjectIdentifier digestAlgOID = request.getMessageImprintAlgOID();
 
         AlgorithmIdentifier algID = new AlgorithmIdentifier(digestAlgOID, new DERNull());
         MessageImprint      messageImprint = new MessageImprint(algID, request.getMessageImprintDigest());
@@ -349,48 +362,48 @@ public class TimeStampTokenGenerator
         Accuracy accuracy = null;
         if (accuracySeconds > 0 || accuracyMillis > 0 || accuracyMicros > 0)
         {
-            DERInteger seconds = null;
+            ASN1Integer seconds = null;
             if (accuracySeconds > 0)
             {
-                seconds = new DERInteger(accuracySeconds);
+                seconds = new ASN1Integer(accuracySeconds);
             }
 
-            DERInteger millis = null;
+            ASN1Integer millis = null;
             if (accuracyMillis > 0)
             {
-                millis = new DERInteger(accuracyMillis);
+                millis = new ASN1Integer(accuracyMillis);
             }
 
-            DERInteger micros = null;
+            ASN1Integer micros = null;
             if (accuracyMicros > 0)
             {
-                micros = new DERInteger(accuracyMicros);
+                micros = new ASN1Integer(accuracyMicros);
             }
 
             accuracy = new Accuracy(seconds, millis, micros);
         }
 
-        DERBoolean derOrdering = null;
+        ASN1Boolean derOrdering = null;
         if (ordering)
         {
-            derOrdering = new DERBoolean(ordering);
+            derOrdering = new ASN1Boolean(ordering);
         }
 
-        DERInteger  nonce = null;
+        ASN1Integer  nonce = null;
         if (request.getNonce() != null)
         {
-            nonce = new DERInteger(request.getNonce());
+            nonce = new ASN1Integer(request.getNonce());
         }
 
-        ASN1ObjectIdentifier tsaPolicy = new ASN1ObjectIdentifier(tsaPolicyOID);
+        ASN1ObjectIdentifier tsaPolicy = tsaPolicyOID;
         if (request.getReqPolicy() != null)
         {
-            tsaPolicy = new ASN1ObjectIdentifier(request.getReqPolicy());
+            tsaPolicy = request.getReqPolicy();
         }
 
         TSTInfo tstInfo = new TSTInfo(tsaPolicy,
-                messageImprint, new DERInteger(serialNumber),
-                new DERGeneralizedTime(genTime), accuracy, derOrdering,
+                messageImprint, new ASN1Integer(serialNumber),
+                new ASN1GeneralizedTime(genTime), accuracy, derOrdering,
                 nonce, tsa, request.getExtensions());
 
         try
@@ -411,7 +424,7 @@ public class TimeStampTokenGenerator
 
             signedDataGenerator.addSignerInfoGenerator(signerInfoGen);
 
-            byte[] derEncodedTSTInfo = tstInfo.getEncoded(ASN1Encodable.DER);
+            byte[] derEncodedTSTInfo = tstInfo.getEncoded(ASN1Encoding.DER);
 
             CMSSignedData signedData = signedDataGenerator.generate(new CMSProcessableByteArray(PKCSObjectIdentifiers.id_ct_TSTInfo, derEncodedTSTInfo), true);
 
