@@ -1,11 +1,13 @@
 package org.bouncycastle.asn1.crmf;
 
-import org.bouncycastle.asn1.ASN1Encodable;
+import org.bouncycastle.asn1.ASN1EncodableVector;
+import org.bouncycastle.asn1.ASN1Object;
+import org.bouncycastle.asn1.ASN1Primitive;
 import org.bouncycastle.asn1.ASN1Sequence;
-import org.bouncycastle.asn1.DERObject;
+import org.bouncycastle.asn1.DERSequence;
 
 public class CertReqMessages
-    extends ASN1Encodable
+    extends ASN1Object
 {
     private ASN1Sequence content;
 
@@ -21,12 +23,28 @@ public class CertReqMessages
             return (CertReqMessages)o;
         }
 
-        if (o instanceof ASN1Sequence)
+        if (o != null)
         {
-            return new CertReqMessages((ASN1Sequence)o);
+            return new CertReqMessages(ASN1Sequence.getInstance(o));
         }
 
-        throw new IllegalArgumentException("Invalid object: " + o.getClass().getName());
+        return null;
+    }
+
+    public CertReqMessages(
+        CertReqMsg msg)
+    {
+        content = new DERSequence(msg);
+    }
+
+    public CertReqMessages(
+        CertReqMsg[] msgs)
+    {
+        ASN1EncodableVector v = new ASN1EncodableVector();
+        for (int i = 0; i < msgs.length; i++) {
+            v.add(msgs[i]);
+        }
+        content = new DERSequence(v);
     }
 
     public CertReqMsg[] toCertReqMsgArray()
@@ -47,7 +65,7 @@ public class CertReqMessages
      * </pre>
      * @return a basic ASN.1 object representation.
      */
-    public DERObject toASN1Object()
+    public ASN1Primitive toASN1Primitive()
     {
         return content;
     }
