@@ -1,11 +1,19 @@
 package org.bouncycastle.asn1.test;
 
+import java.io.ByteArrayInputStream;
+import java.io.ByteArrayOutputStream;
+import java.io.IOException;
+import java.util.Hashtable;
+import java.util.Vector;
+
+import org.bouncycastle.asn1.ASN1Encodable;
 import org.bouncycastle.asn1.ASN1EncodableVector;
 import org.bouncycastle.asn1.ASN1InputStream;
+import org.bouncycastle.asn1.ASN1ObjectIdentifier;
 import org.bouncycastle.asn1.ASN1OutputStream;
+import org.bouncycastle.asn1.ASN1Primitive;
 import org.bouncycastle.asn1.ASN1Sequence;
 import org.bouncycastle.asn1.ASN1Set;
-import org.bouncycastle.asn1.DEREncodable;
 import org.bouncycastle.asn1.DERGeneralizedTime;
 import org.bouncycastle.asn1.DERIA5String;
 import org.bouncycastle.asn1.DERObjectIdentifier;
@@ -13,18 +21,15 @@ import org.bouncycastle.asn1.DERPrintableString;
 import org.bouncycastle.asn1.DERSequence;
 import org.bouncycastle.asn1.DERSet;
 import org.bouncycastle.asn1.DERUTF8String;
+import org.bouncycastle.asn1.x500.X500Name;
+import org.bouncycastle.asn1.x500.X500NameBuilder;
+import org.bouncycastle.asn1.x500.style.BCStyle;
 import org.bouncycastle.asn1.x509.X509DefaultEntryConverter;
 import org.bouncycastle.asn1.x509.X509Name;
 import org.bouncycastle.util.Arrays;
 import org.bouncycastle.util.encoders.Hex;
 import org.bouncycastle.util.test.SimpleTest;
 
-import java.io.ByteArrayInputStream;
-import java.io.ByteArrayOutputStream;
-import java.io.IOException;
-import java.util.Hashtable;
-import java.util.Vector;
-
 public class X509NameTest
     extends SimpleTest
 {
@@ -51,30 +56,38 @@ public class X509NameTest
         return X509Name.getInstance(new ASN1InputStream(new ByteArrayInputStream(bytes)).readObject());
     }
 
-    private DEREncodable createEntryValue(DERObjectIdentifier oid, String value)
+    private ASN1Encodable createEntryValue(DERObjectIdentifier oid, String value)
     {
         Hashtable attrs = new Hashtable();
 
         attrs.put(oid, value);
 
-        X509Name name = new X509Name(attrs);
+        Vector order = new Vector();
 
-        ASN1Sequence seq = (ASN1Sequence)name.getDERObject();
+        order.addElement(oid);
+        
+        X509Name name = new X509Name(order, attrs);
+
+        ASN1Sequence seq = (ASN1Sequence)name.toASN1Primitive();
         ASN1Set set = (ASN1Set)seq.getObjectAt(0);
         seq = (ASN1Sequence)set.getObjectAt(0);
 
         return seq.getObjectAt(1);
     }
 
-    private DEREncodable createEntryValueFromString(DERObjectIdentifier oid, String value)
+    private ASN1Encodable createEntryValueFromString(DERObjectIdentifier oid, String value)
     {
         Hashtable attrs = new Hashtable();
 
         attrs.put(oid, value);
 
-        X509Name name = new X509Name(new X509Name(attrs).toString());
+        Vector order = new Vector();
 
-        ASN1Sequence seq = (ASN1Sequence)name.getDERObject();
+        order.addElement(oid);
+        
+        X509Name name = new X509Name(new X509Name(order, attrs).toString());
+
+        ASN1Sequence seq = (ASN1Sequence)name.toASN1Primitive();
         ASN1Set set = (ASN1Set)seq.getObjectAt(0);
         seq = (ASN1Sequence)set.getObjectAt(0);
 
@@ -83,7 +96,7 @@ public class X509NameTest
 
     private void testEncodingPrintableString(DERObjectIdentifier oid, String value)
     {
-        DEREncodable converted = createEntryValue(oid, value);
+        ASN1Encodable converted = createEntryValue(oid, value);
         if (!(converted instanceof DERPrintableString))
         {
             fail("encoding for " + oid + " not printable string");
@@ -92,7 +105,7 @@ public class X509NameTest
 
     private void testEncodingIA5String(DERObjectIdentifier oid, String value)
     {
-        DEREncodable converted = createEntryValue(oid, value);
+        ASN1Encodable converted = createEntryValue(oid, value);
         if (!(converted instanceof DERIA5String))
         {
             fail("encoding for " + oid + " not IA5String");
@@ -102,7 +115,7 @@ public class X509NameTest
 
     private void testEncodingGeneralizedTime(DERObjectIdentifier oid, String value)
     {
-        DEREncodable converted = createEntryValue(oid, value);
+        ASN1Encodable converted = createEntryValue(oid, value);
         if (!(converted instanceof DERGeneralizedTime))
         {
             fail("encoding for " + oid + " not GeneralizedTime");
@@ -138,7 +151,15 @@ public class X509NameTest
         attrs.put(X509Name.ST, "Victoria");
         attrs.put(X509Name.E, "feedback-crypto@bouncycastle.org");
 
-        X509Name    name1 = new X509Name(attrs);
+        Vector                     order = new Vector();
+
+        order.addElement(X509Name.C);
+        order.addElement(X509Name.O);
+        order.addElement(X509Name.L);
+        order.addElement(X509Name.ST);
+        order.addElement(X509Name.E);
+
+        X509Name    name1 = new X509Name(order, attrs);
 
         if (!name1.equals(name1))
         {
@@ -150,7 +171,7 @@ public class X509NameTest
             fail("Failed same object test - in Order");
         }
 
-        X509Name    name2 = new X509Name(attrs);
+        X509Name    name2 = new X509Name(order, attrs);
 
         if (!name1.equals(name2))
         {
@@ -191,6 +212,11 @@ public class X509NameTest
             fail("Failed reverse name test");
         }
 
+        if (name1.hashCode() != name2.hashCode())
+        {
+            fail("Failed reverse name test hashCode");
+        }
+
         if (name1.equals(name2, true))
         {
             fail("Failed reverse name test - in Order");
@@ -256,7 +282,6 @@ public class X509NameTest
 
         compositeTest();
 
-
         ByteArrayOutputStream bOut;
         ASN1OutputStream aOut;
         ASN1InputStream aIn;
@@ -340,6 +365,38 @@ public class X509NameTest
         equalityTest(new X509Name("CN=  The     Legion "), new X509Name("CN=The Legion"));
         equalityTest(new X509Name("CN=  the     legion "), new X509Name("CN=The Legion"));
 
+        // # test
+
+        X509Name n1 = new X509Name("SERIALNUMBER=8,O=ABC,CN=ABC Class 3 CA,C=LT");
+        X509Name n2 = new X509Name("2.5.4.5=8,O=ABC,CN=ABC Class 3 CA,C=LT");
+        X509Name n3 = new X509Name("2.5.4.5=#130138,O=ABC,CN=ABC Class 3 CA,C=LT");
+
+        equalityTest(n1, n2);
+        equalityTest(n2, n3);
+        equalityTest(n3, n1);
+
+        n1 = new X509Name(true, "2.5.4.5=#130138,CN=SSC Class 3 CA,O=UAB Skaitmeninio sertifikavimo centras,C=LT");
+        n2 = new X509Name(true, "SERIALNUMBER=#130138,CN=SSC Class 3 CA,O=UAB Skaitmeninio sertifikavimo centras,C=LT");
+        n3 = X509Name.getInstance(ASN1Primitive.fromByteArray(Hex.decode("3063310b3009060355040613024c54312f302d060355040a1326"
+            + "55414220536b6169746d656e696e696f20736572746966696b6176696d6f2063656e74726173311730150603550403130e53534320436c6173732033204341310a30080603550405130138")));
+
+        equalityTest(n1, n2);
+        equalityTest(n2, n3);
+        equalityTest(n3, n1);
+
+        n1 = new X509Name("SERIALNUMBER=8,O=XX,CN=ABC Class 3 CA,C=LT");
+        n2 = new X509Name("2.5.4.5=8,O=,CN=ABC Class 3 CA,C=LT");
+
+        if (n1.equals(n2))
+        {
+            fail("empty inequality check failed");
+        }
+
+        n1 = new X509Name("SERIALNUMBER=8,O=,CN=ABC Class 3 CA,C=LT");
+        n2 = new X509Name("2.5.4.5=8,O=,CN=ABC Class 3 CA,C=LT");
+
+        equalityTest(n1, n2);
+        
         //
         // inequality to sequences
         //
@@ -423,8 +480,6 @@ public class X509NameTest
         }
         */
 
-
-
         DERUTF8String testString = new DERUTF8String("The Legion of the Bouncy Castle");
         byte[] encodedBytes = testString.getEncoded();
         byte[] hexEncodedBytes = Hex.encode(encodedBytes);
@@ -466,6 +521,66 @@ public class X509NameTest
         {
             fail("escaped # not reduced properly");
         }
+
+        n = new X509Name("CN=\"a+b\"");
+
+        vls = n.getValues(X509Name.CN);
+        if (vls.size() != 1 || !vls.elementAt(0).equals("a+b"))
+        {
+            fail("escaped + not reduced properly");
+        }
+
+        n = new X509Name("CN=a\\+b");
+
+        vls = n.getValues(X509Name.CN);
+        if (vls.size() != 1 || !vls.elementAt(0).equals("a+b"))
+        {
+            fail("escaped + not reduced properly");
+        }
+
+        if (!n.toString().equals("CN=a\\+b"))
+        {
+            fail("+ in string not properly escaped.");
+        }
+
+        n = new X509Name("CN=a\\=b");
+
+        vls = n.getValues(X509Name.CN);
+        if (vls.size() != 1 || !vls.elementAt(0).equals("a=b"))
+        {
+            fail("escaped = not reduced properly");
+        }
+
+        if (!n.toString().equals("CN=a\\=b"))
+        {
+            fail("= in string not properly escaped.");
+        }
+
+        n = new X509Name("TELEPHONENUMBER=\"+61999999999\"");
+
+        vls = n.getValues(X509Name.TELEPHONE_NUMBER);
+        if (vls.size() != 1 || !vls.elementAt(0).equals("+61999999999"))
+        {
+            fail("telephonenumber escaped + not reduced properly");
+        }
+
+        n = new X509Name("TELEPHONENUMBER=\\+61999999999");
+
+        vls = n.getValues(X509Name.TELEPHONE_NUMBER);
+        if (vls.size() != 1 || !vls.elementAt(0).equals("+61999999999"))
+        {
+            fail("telephonenumber escaped + not reduced properly");
+        }
+
+        // migration
+        X500NameBuilder builder = new X500NameBuilder(BCStyle.INSTANCE);
+        builder.addMultiValuedRDN(new ASN1ObjectIdentifier[] { BCStyle.CN, BCStyle.SN }, new String[] { "Thomas", "CVR:12341233-UID:1111" });
+        builder.addRDN(BCStyle.O, "Test");
+        builder.addRDN(BCStyle.C, "DK");
+
+        X500Name subject = builder.build();
+        ASN1Primitive derObject = subject.toASN1Primitive();
+        X509Name instance = X509Name.getInstance(derObject);
     }
 
     private boolean compareVectors(Vector a, Vector b)    // for compatibility with early JDKs
@@ -526,6 +641,13 @@ public class X509NameTest
         {
             fail("Failed composite string to encoding test");
         }
+        
+        //
+        // dud name test - handle empty DN without barfing.
+        //
+        n = new X509Name("C=CH,O=,OU=dummy,CN=mail@dummy.com");
+
+        n = X509Name.getInstance(ASN1Primitive.fromByteArray(n.getEncoded()));
     }
 
     private void equalityTest(X509Name x509Name, X509Name x509Name1)
@@ -535,6 +657,11 @@ public class X509NameTest
             fail("equality test failed for " + x509Name + " : " + x509Name1);
         }
 
+        if (x509Name.hashCode() != x509Name1.hashCode())
+        {
+            fail("hashCodeTest test failed for " + x509Name + " : " + x509Name1);
+        }
+
         if (!x509Name.equals(x509Name1, true))
         {
             fail("equality test failed for " + x509Name + " : " + x509Name1);
