@@ -1,5 +1,25 @@
 package org.bouncycastle.cms.test;
 
+import java.io.IOException;
+import java.security.GeneralSecurityException;
+import java.security.Key;
+import java.security.KeyFactory;
+import java.security.KeyPair;
+import java.security.NoSuchAlgorithmException;
+import java.security.NoSuchProviderException;
+import java.security.PrivateKey;
+import java.security.Security;
+import java.security.cert.CertificateEncodingException;
+import java.security.cert.X509Certificate;
+import java.security.spec.PKCS8EncodedKeySpec;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Collection;
+import java.util.Iterator;
+
+import javax.crypto.SecretKey;
+import javax.crypto.spec.SecretKeySpec;
+
 import junit.framework.Test;
 import junit.framework.TestCase;
 import junit.framework.TestSuite;
@@ -18,32 +38,21 @@ import org.bouncycastle.cms.CMSPBEKey;
 import org.bouncycastle.cms.CMSProcessableByteArray;
 import org.bouncycastle.cms.KeyTransRecipientInformation;
 import org.bouncycastle.cms.PKCS5Scheme2PBEKey;
-import org.bouncycastle.cms.RecipientInformation;
-import org.bouncycastle.cms.RecipientInformationStore;
 import org.bouncycastle.cms.PKCS5Scheme2UTF8PBEKey;
 import org.bouncycastle.cms.PasswordRecipientInformation;
+import org.bouncycastle.cms.RecipientId;
+import org.bouncycastle.cms.RecipientInformation;
+import org.bouncycastle.cms.RecipientInformationStore;
+import org.bouncycastle.cms.jcajce.JceKeyAgreeRecipientId;
+import org.bouncycastle.jce.provider.BouncyCastleProvider;
 import org.bouncycastle.util.encoders.Base64;
 import org.bouncycastle.util.encoders.Hex;
 
-import javax.crypto.SecretKey;
-import javax.crypto.spec.SecretKeySpec;
-import java.security.GeneralSecurityException;
-import java.security.Key;
-import java.security.KeyFactory;
-import java.security.KeyPair;
-import java.security.NoSuchAlgorithmException;
-import java.security.NoSuchProviderException;
-import java.security.PrivateKey;
-import java.security.Security;
-import java.security.cert.X509Certificate;
-import java.security.spec.PKCS8EncodedKeySpec;
-import java.util.Arrays;
-import java.util.Collection;
-import java.util.Iterator;
-
 public class EnvelopedDataTest
     extends TestCase 
 {
+    private static final String BC = BouncyCastleProvider.PROVIDER_NAME;
+
     private static String          _signDN;
     private static KeyPair         _signKP;  
     private static X509Certificate _signCert;
@@ -53,12 +62,15 @@ public class EnvelopedDataTest
     private static X509Certificate _origCert;
 
     private static String          _reciDN;
+    private static String          _reciDN2;
     private static KeyPair         _reciKP;
     private static X509Certificate _reciCert;
 
     private static KeyPair         _origEcKP;
     private static KeyPair         _reciEcKP;
     private static X509Certificate _reciEcCert;
+    private static KeyPair         _reciEcKP2;
+    private static X509Certificate _reciEcCert2;
 
     private static boolean         _initialised = false;
 
@@ -148,7 +160,7 @@ public class EnvelopedDataTest
          + "HnJVHsQ6X56VcwYb+OfojTBJBgkqhkiG9w0BBwEwGgYIKoZIhvcNAwIwDgIC"
          + "AKAECJwE0hkuKlWhgCBeKNXhojuej3org9Lt7n+wWxOhnky5V50vSpoYRfRR"
          + "yw==");
-
+    
     public EnvelopedDataTest()
     {
     }
@@ -169,12 +181,15 @@ public class EnvelopedDataTest
             _origCert = CMSTestUtil.makeCertificate(_origKP, _origDN, _signKP, _signDN);
 
             _reciDN   = "CN=Doug, OU=Sales, O=Bouncy Castle, C=AU";
+            _reciDN2  = "CN=Fred, OU=Sales, O=Bouncy Castle, C=AU";
             _reciKP   = CMSTestUtil.makeKeyPair();
             _reciCert = CMSTestUtil.makeCertificate(_reciKP, _reciDN, _signKP, _signDN);
 
             _origEcKP = CMSTestUtil.makeEcDsaKeyPair();
             _reciEcKP = CMSTestUtil.makeEcDsaKeyPair();
             _reciEcCert = CMSTestUtil.makeCertificate(_reciEcKP, _reciDN, _signKP, _signDN);
+            _reciEcKP2 = CMSTestUtil.makeEcDsaKeyPair();
+            _reciEcCert2 = CMSTestUtil.makeCertificate(_reciEcKP2, _reciDN2, _signKP, _signDN);
         }
     }
     
@@ -204,7 +219,7 @@ public class EnvelopedDataTest
 
         CMSEnvelopedData ed = edGen.generate(
                                 new CMSProcessableByteArray(data),
-                                CMSEnvelopedDataGenerator.DES_EDE3_CBC, "BC");
+                                CMSEnvelopedDataGenerator.DES_EDE3_CBC, BC);
 
         RecipientInformationStore  recipients = ed.getRecipientInfos();
 
@@ -223,7 +238,7 @@ public class EnvelopedDataTest
 
             assertEquals(recipient.getKeyEncryptionAlgOID(), PKCSObjectIdentifiers.rsaEncryption.getId());
             
-            byte[] recData = recipient.getContent(_reciKP.getPrivate(), "BC");
+            byte[] recData = recipient.getContent(_reciKP.getPrivate(), BC);
 
             assertEquals(true, Arrays.equals(data, recData));
         }
@@ -285,7 +300,7 @@ public class EnvelopedDataTest
 
         CMSEnvelopedData ed = edGen.generate(
                                 new CMSProcessableByteArray(data),
-                                "1.2.840.113549.3.4", "BC");
+                                "1.2.840.113549.3.4", BC);
 
         RecipientInformationStore  recipients = ed.getRecipientInfos();
 
@@ -301,7 +316,7 @@ public class EnvelopedDataTest
         {
             RecipientInformation   recipient = (RecipientInformation)it.next();
 
-            byte[] recData = recipient.getContent(_reciKP.getPrivate(), "BC");
+            byte[] recData = recipient.getContent(_reciKP.getPrivate(), BC);
 
             assertEquals(true, Arrays.equals(data, recData));
         }
@@ -318,7 +333,7 @@ public class EnvelopedDataTest
 
         CMSEnvelopedData ed = edGen.generate(
                                 new CMSProcessableByteArray(data),
-                                "1.2.840.113549.3.4", 128, "BC");
+                                "1.2.840.113549.3.4", 128, BC);
 
         RecipientInformationStore  recipients = ed.getRecipientInfos();
 
@@ -331,7 +346,7 @@ public class EnvelopedDataTest
         {
             RecipientInformation   recipient = (RecipientInformation)it.next();
 
-            byte[] recData = recipient.getContent(_reciKP.getPrivate(), "BC");
+            byte[] recData = recipient.getContent(_reciKP.getPrivate(), BC);
 
             assertEquals(true, Arrays.equals(data, recData));
         }
@@ -352,7 +367,7 @@ public class EnvelopedDataTest
 
         CMSEnvelopedData ed = edGen.generate(
                                 new CMSProcessableByteArray(data),
-                                "1.3.14.3.2.7", "BC");
+                                "1.3.14.3.2.7", BC);
 
         RecipientInformationStore  recipients = ed.getRecipientInfos();
 
@@ -365,7 +380,7 @@ public class EnvelopedDataTest
         {
             RecipientInformation   recipient = (RecipientInformation)it.next();
 
-            byte[] recData = recipient.getContent(_reciKP.getPrivate(), "BC");
+            byte[] recData = recipient.getContent(_reciKP.getPrivate(), BC);
 
             assertEquals(true, Arrays.equals(data, recData));
         }
@@ -386,7 +401,7 @@ public class EnvelopedDataTest
 
         CMSEnvelopedData ed = edGen.generate(
                               new CMSProcessableByteArray(data),
-                              CMSEnvelopedDataGenerator.AES128_CBC, "BC");
+                              CMSEnvelopedDataGenerator.AES128_CBC, BC);
 
         RecipientInformationStore  recipients = ed.getRecipientInfos();
 
@@ -400,7 +415,7 @@ public class EnvelopedDataTest
         {
             RecipientInformation   recipient = (RecipientInformation)it.next();
 
-            byte[] recData = recipient.getContent(_reciKP.getPrivate(), "BC");
+            byte[] recData = recipient.getContent(_reciKP.getPrivate(), BC);
             assertEquals(true, Arrays.equals(data, recData));
         }
         else
@@ -468,7 +483,7 @@ public class EnvelopedDataTest
 
         CMSEnvelopedData ed = edGen.generate(
                                 new CMSProcessableByteArray(data),
-                                generatorOID, "BC");
+                                generatorOID, BC);
 
         RecipientInformationStore  recipients = ed.getRecipientInfos();
 
@@ -498,7 +513,7 @@ public class EnvelopedDataTest
 
             assertEquals(recipient.getKeyEncryptionAlgOID(), PKCSObjectIdentifiers.rsaEncryption.getId());
 
-            byte[] recData = recipient.getContent(_reciKP.getPrivate(), "BC");
+            byte[] recData = recipient.getContent(_reciKP.getPrivate(), BC);
 
             assertEquals(true, Arrays.equals(data, recData));
         }
@@ -525,7 +540,7 @@ public class EnvelopedDataTest
 
             assertEquals(recipient.getKeyEncryptionAlgOID(), NISTObjectIdentifiers.id_aes128_wrap.getId());
 
-            byte[] recData = recipient.getContent(kek, "BC");
+            byte[] recData = recipient.getContent(kek, BC);
 
             assertEquals(true, Arrays.equals(data, recData));
         }
@@ -600,7 +615,7 @@ public class EnvelopedDataTest
 
         CMSEnvelopedData ed = edGen.generate(
                                 new CMSProcessableByteArray(data),
-                                CMSEnvelopedDataGenerator.DES_EDE3_CBC, "BC");
+                                CMSEnvelopedDataGenerator.DES_EDE3_CBC, BC);
 
         RecipientInformationStore recipients = ed.getRecipientInfos();
 
@@ -615,7 +630,7 @@ public class EnvelopedDataTest
 
             assertEquals(algOid.getId(), recipient.getKeyEncryptionAlgOID());
 
-            byte[] recData = recipient.getContent(kek, "BC");
+            byte[] recData = recipient.getContent(kek, BC);
 
             assertTrue(Arrays.equals(data, recData));
         }
@@ -632,55 +647,113 @@ public class EnvelopedDataTest
 
         CMSEnvelopedDataGenerator edGen = new CMSEnvelopedDataGenerator();
 
-        edGen.addKeyAgreementRecipient(CMSEnvelopedDataGenerator.ECDH_SHA1KDF, _origEcKP.getPrivate(), _origEcKP.getPublic(), _reciEcCert, CMSEnvelopedDataGenerator.AES128_WRAP, "BC");
+        edGen.addKeyAgreementRecipient(CMSEnvelopedDataGenerator.ECDH_SHA1KDF,
+            _origEcKP.getPrivate(), _origEcKP.getPublic(),
+            _reciEcCert, CMSEnvelopedDataGenerator.AES128_WRAP, BC);
 
         CMSEnvelopedData ed = edGen.generate(
-                              new CMSProcessableByteArray(data),
-                              CMSEnvelopedDataGenerator.AES128_CBC, "BC");
+            new CMSProcessableByteArray(data),
+            CMSEnvelopedDataGenerator.AES128_CBC, BC);
 
-        RecipientInformationStore  recipients = ed.getRecipientInfos();
+        assertEquals(ed.getEncryptionAlgOID(), CMSEnvelopedDataGenerator.AES128_CBC);
 
-        assertEquals(ed.getEncryptionAlgOID(),
-                                   CMSEnvelopedDataGenerator.AES128_CBC);
+        RecipientInformationStore recipients = ed.getRecipientInfos();
 
-        Collection  c = recipients.getRecipients();
-        Iterator    it = c.iterator();
+        confirmDataReceived(recipients, data, _reciEcCert, _reciEcKP.getPrivate(), BC);
+        confirmNumberRecipients(recipients, 1);
+    }
 
-        if (it.hasNext())
-        {
-            RecipientInformation   recipient = (RecipientInformation)it.next();
+    public void testECMQVKeyAgree()
+        throws Exception
+    {
+        byte[] data = Hex.decode("504b492d4320434d5320456e76656c6f706564446174612053616d706c65");
 
-            byte[] recData = recipient.getContent(_reciEcKP.getPrivate(), "BC");
-            assertEquals(true, Arrays.equals(data, recData));
-        }
-        else
-        {
-            fail("no recipient found");
-        }
+        CMSEnvelopedDataGenerator edGen = new CMSEnvelopedDataGenerator();
+
+        edGen.addKeyAgreementRecipient(CMSEnvelopedDataGenerator.ECMQV_SHA1KDF,
+            _origEcKP.getPrivate(), _origEcKP.getPublic(),
+            _reciEcCert, CMSEnvelopedDataGenerator.AES128_WRAP, BC);
+
+        CMSEnvelopedData ed = edGen.generate(
+            new CMSProcessableByteArray(data),
+            CMSEnvelopedDataGenerator.AES128_CBC, BC);
+
+        assertEquals(ed.getEncryptionAlgOID(), CMSEnvelopedDataGenerator.AES128_CBC);
+
+        RecipientInformationStore recipients = ed.getRecipientInfos();
+
+        confirmDataReceived(recipients, data, _reciEcCert, _reciEcKP.getPrivate(), BC);
+        confirmNumberRecipients(recipients, 1);
+    }
+
+    public void testECMQVKeyAgreeMultiple()
+        throws Exception
+    {
+        byte[] data = Hex.decode("504b492d4320434d5320456e76656c6f706564446174612053616d706c65");
+
+        CMSEnvelopedDataGenerator edGen = new CMSEnvelopedDataGenerator();
+
+        ArrayList recipientCerts = new ArrayList();
+        recipientCerts.add(_reciEcCert);
+        recipientCerts.add(_reciEcCert2);
+
+        edGen.addKeyAgreementRecipients(CMSEnvelopedDataGenerator.ECMQV_SHA1KDF,
+            _origEcKP.getPrivate(), _origEcKP.getPublic(),
+            recipientCerts, CMSEnvelopedDataGenerator.AES128_WRAP, BC);
+
+        CMSEnvelopedData ed = edGen.generate(
+            new CMSProcessableByteArray(data),
+            CMSEnvelopedDataGenerator.AES128_CBC, BC);
+
+        assertEquals(ed.getEncryptionAlgOID(), CMSEnvelopedDataGenerator.AES128_CBC);
+
+        RecipientInformationStore recipients = ed.getRecipientInfos();
+
+        confirmDataReceived(recipients, data, _reciEcCert, _reciEcKP.getPrivate(), BC);
+        confirmDataReceived(recipients, data, _reciEcCert2, _reciEcKP2.getPrivate(), BC);
+        confirmNumberRecipients(recipients, 2);
+    }
+
+    private static void confirmDataReceived(RecipientInformationStore recipients,
+        byte[] expectedData, X509Certificate reciCert, PrivateKey reciPrivKey, String provider)
+        throws CMSException, NoSuchProviderException, CertificateEncodingException, IOException
+    {
+        RecipientId rid = new JceKeyAgreeRecipientId(reciCert);
+
+        RecipientInformation recipient = recipients.get(rid);
+        assertNotNull(recipient);
+
+        byte[] actualData = recipient.getContent(reciPrivKey, provider);
+        assertEquals(true, Arrays.equals(expectedData, actualData));
+    }
+
+    private static void confirmNumberRecipients(RecipientInformationStore recipients, int count)
+    {
+        assertEquals(count, recipients.getRecipients().size());
     }
 
     public void testECKeyAgreeVectors()
         throws Exception
     {
         PKCS8EncodedKeySpec privSpec = new PKCS8EncodedKeySpec(ecKeyAgreeKey);
-        KeyFactory          fact = KeyFactory.getInstance("ECDH", "BC");
+        KeyFactory          fact = KeyFactory.getInstance("ECDH", BC);
         PrivateKey          privKey = fact.generatePrivate(privSpec);
 
         verifyECKeyAgreeVectors(privKey, "2.16.840.1.101.3.4.1.42", ecKeyAgreeMsgAES256);
         verifyECKeyAgreeVectors(privKey, "2.16.840.1.101.3.4.1.2", ecKeyAgreeMsgAES128);
         verifyECKeyAgreeVectors(privKey, "1.2.840.113549.3.7", ecKeyAgreeMsgDESEDE);
     }
-    /*
+
     public void testECMQVKeyAgreeVectors()
         throws Exception
     {
         PKCS8EncodedKeySpec privSpec = new PKCS8EncodedKeySpec(ecKeyAgreeKey);
-        KeyFactory          fact = KeyFactory.getInstance("ECDH", "BC");
+        KeyFactory          fact = KeyFactory.getInstance("ECDH", BC);
         PrivateKey          privKey = fact.generatePrivate(privSpec);
 
         verifyECMQVKeyAgreeVectors(privKey, "2.16.840.1.101.3.4.1.2", ecMQVKeyAgreeMsgAES128);
     }
-    */
+
     public void testPasswordAES256()
         throws Exception
     {
@@ -700,7 +773,7 @@ public class EnvelopedDataTest
     {
         byte[] data = Hex.decode("5468697320697320736f6d652073616d706c6520636f6e74656e742e");
 
-        KeyFactory kFact = KeyFactory.getInstance("RSA", "BC");
+        KeyFactory kFact = KeyFactory.getInstance("RSA", BC);
         Key key = kFact.generatePrivate(new PKCS8EncodedKeySpec(bobPrivRsaEncrypt));
 
         CMSEnvelopedData ed = new CMSEnvelopedData(rfc4134ex5_1);
@@ -716,7 +789,7 @@ public class EnvelopedDataTest
         {
             RecipientInformation   recipient = (RecipientInformation)it.next();
 
-            byte[] recData = recipient.getContent(key, "BC");
+            byte[] recData = recipient.getContent(key, BC);
 
             assertEquals(true, Arrays.equals(data, recData));
         }
@@ -731,7 +804,7 @@ public class EnvelopedDataTest
     {
         byte[] data = Hex.decode("5468697320697320736f6d652073616d706c6520636f6e74656e742e");
 
-        KeyFactory kFact = KeyFactory.getInstance("RSA", "BC");
+        KeyFactory kFact = KeyFactory.getInstance("RSA", BC);
         Key key = kFact.generatePrivate(new PKCS8EncodedKeySpec(bobPrivRsaEncrypt));
 
         CMSEnvelopedData ed = new CMSEnvelopedData(rfc4134ex5_2);
@@ -752,7 +825,7 @@ public class EnvelopedDataTest
 
                 if (recipient instanceof KeyTransRecipientInformation)
                 {
-                    recData = recipient.getContent(key, "BC");
+                    recData = recipient.getContent(key, BC);
 
                     assertEquals(true, Arrays.equals(data, recData));
                 }
@@ -786,7 +859,7 @@ public class EnvelopedDataTest
 
         CMSEnvelopedData ed = edGen.generate(
                               new CMSProcessableByteArray(data),
-                              CMSEnvelopedDataGenerator.AES128_CBC, "BC");
+                              CMSEnvelopedDataGenerator.AES128_CBC, BC);
 
         RecipientInformationStore  recipients = ed.getRecipientInfos();
 
@@ -801,9 +874,9 @@ public class EnvelopedDataTest
             PasswordRecipientInformation recipient = (PasswordRecipientInformation)it.next();
 
             CMSPBEKey key = new PKCS5Scheme2PBEKey("password".toCharArray(),
-                recipient.getKeyDerivationAlgParameters("BC"));
+                recipient.getKeyDerivationAlgParameters(BC));
 
-            byte[] recData = recipient.getContent(key, "BC");
+            byte[] recData = recipient.getContent(key, BC);
 
             assertEquals(true, Arrays.equals(data, recData));
         }
@@ -819,7 +892,7 @@ public class EnvelopedDataTest
 
         RecipientInformation   recipient = (RecipientInformation)it.next();
 
-        byte[] recData = recipient.getContent(new PKCS5Scheme2PBEKey("password".toCharArray(), ((PasswordRecipientInformation)recipient).getKeyDerivationAlgParameters("BC")), "BC");
+        byte[] recData = recipient.getContent(new PKCS5Scheme2PBEKey("password".toCharArray(), ((PasswordRecipientInformation)recipient).getKeyDerivationAlgParameters(BC)), BC);
         assertEquals(true, Arrays.equals(data, recData));
     }
 
@@ -834,7 +907,7 @@ public class EnvelopedDataTest
 
         CMSEnvelopedData ed = edGen.generate(
                               new CMSProcessableByteArray(data),
-                              CMSEnvelopedDataGenerator.AES128_CBC, "BC");
+                              CMSEnvelopedDataGenerator.AES128_CBC, BC);
 
         RecipientInformationStore  recipients = ed.getRecipientInfos();
 
@@ -848,7 +921,7 @@ public class EnvelopedDataTest
         {
             RecipientInformation   recipient = (RecipientInformation)it.next();
 
-            byte[] recData = recipient.getContent(new PKCS5Scheme2UTF8PBEKey("abc\u5639\u563b".toCharArray(), new byte[20], 5), "BC");
+            byte[] recData = recipient.getContent(new PKCS5Scheme2UTF8PBEKey("abc\u5639\u563b".toCharArray(), new byte[20], 5), BC);
             assertEquals(true, Arrays.equals(data, recData));
         }
         else
@@ -863,7 +936,7 @@ public class EnvelopedDataTest
 
         RecipientInformation   recipient = (RecipientInformation)it.next();
 
-        byte[] recData = recipient.getContent(new PKCS5Scheme2UTF8PBEKey("abc\u5639\u563b".toCharArray(), ((PasswordRecipientInformation)recipient).getKeyDerivationAlgParameters("BC")), "BC");
+        byte[] recData = recipient.getContent(new PKCS5Scheme2UTF8PBEKey("abc\u5639\u563b".toCharArray(), ((PasswordRecipientInformation)recipient).getKeyDerivationAlgParameters(BC)), BC);
         assertEquals(true, Arrays.equals(data, recData));
     }
 
@@ -887,7 +960,7 @@ public class EnvelopedDataTest
 
             assertEquals("1.3.133.16.840.63.0.2", recipient.getKeyEncryptionAlgOID());
 
-            byte[] recData = recipient.getContent(privKey, "BC");
+            byte[] recData = recipient.getContent(privKey, BC);
 
             assertTrue(Arrays.equals(data, recData));
         }
@@ -917,7 +990,7 @@ public class EnvelopedDataTest
 
             assertEquals("1.3.133.16.840.63.0.16", recipient.getKeyEncryptionAlgOID());
 
-            byte[] recData = recipient.getContent(privKey, "BC");
+            byte[] recData = recipient.getContent(privKey, BC);
 
             assertTrue(Arrays.equals(data, recData));
         }
