@@ -56,6 +56,7 @@ public class RegressionTest
         new SHA256DigestTest(),
         new SHA384DigestTest(),
         new SHA512DigestTest(),
+        new SHA3DigestTest(),
         new RIPEMD128DigestTest(),
         new RIPEMD160DigestTest(),
         new RIPEMD256DigestTest(),
@@ -80,6 +81,7 @@ public class RegressionTest
         new KDF1GeneratorTest(),
         new KDF2GeneratorTest(),
         new MGF1GeneratorTest(),
+        new HKDFGeneratorTest(),
         new DHKEKGeneratorTest(),
         new ECDHKEKGeneratorTest(),
         new ShortenedDigestTest(),
@@ -103,8 +105,10 @@ public class RegressionTest
         new Grain128Test(),
         //new NaccacheSternTest(),
         new SRP6Test(),
+        new SCryptTest(),
         new ResetTest(),
-        new NullTest()
+        new NullTest(),
+        new DSTU4145Test()
     };
 
     public static void main(
