@@ -21,12 +21,12 @@ import java.util.Enumeration;
 import java.util.Hashtable;
 import java.util.Vector;
 
+import org.bouncycastle.asn1.ASN1Encodable;
 import org.bouncycastle.asn1.ASN1InputStream;
 import org.bouncycastle.asn1.ASN1OctetString;
 import org.bouncycastle.asn1.ASN1Sequence;
 import org.bouncycastle.asn1.ASN1StreamParser;
 import org.bouncycastle.asn1.DERBMPString;
-import org.bouncycastle.asn1.DEREncodable;
 import org.bouncycastle.asn1.DERSequenceParser;
 import org.bouncycastle.asn1.pkcs.ContentInfo;
 import org.bouncycastle.asn1.pkcs.EncryptedData;
@@ -499,6 +499,18 @@ public class PKCS12StoreTest
             {
                 pName = n;
             }
+            else
+            {
+                // the store's we're using here are consistent so this test will pass - it's actually
+                // possible for this test to fail in other circumstances as PKCS#12 allows certificates
+                // to be stored multiple times under different aliases.
+                X509Certificate cert = (X509Certificate)store.getCertificate(n);
+
+                if (!store.getCertificateAlias(cert).equals(n))
+                {
+                    fail("certificate alias check fails");
+                }
+            }
         }
 
         PrivateKey key = (PrivateKey)store.getKey(pName, null);
@@ -572,7 +584,7 @@ public class PKCS12StoreTest
             fail("Modulus doesn't match.");
         }
 
-        DEREncodable outer = new ASN1StreamParser(data).readObject();
+        ASN1Encodable outer = new ASN1StreamParser(data).readObject();
         if (!(outer instanceof DERSequenceParser))
         {
             fail("Failed DER encoding test.");
@@ -932,7 +944,7 @@ public class PKCS12StoreTest
         //
         ASN1InputStream aIn = new ASN1InputStream(bOut.toByteArray());
 
-        Pfx pfx = new Pfx((ASN1Sequence)aIn.readObject());
+        Pfx pfx = Pfx.getInstance(aIn.readObject());
 
         ContentInfo cInfo = pfx.getAuthSafe();
 
@@ -946,11 +958,11 @@ public class PKCS12StoreTest
 
         aIn = new ASN1InputStream(((ASN1OctetString)c1.getContent()).getOctets());
 
-        SafeBag sb = new SafeBag((ASN1Sequence)(((ASN1Sequence)aIn.readObject()).getObjectAt(0)));
+        SafeBag sb = SafeBag.getInstance((((ASN1Sequence)aIn.readObject()).getObjectAt(0)));
 
         EncryptedPrivateKeyInfo encInfo = EncryptedPrivateKeyInfo.getInstance(sb.getBagValue());
 
-        if (!encInfo.getEncryptionAlgorithm().getObjectId().equals(PKCSObjectIdentifiers.pbeWithSHAAnd3_KeyTripleDES_CBC))
+        if (!encInfo.getEncryptionAlgorithm().getAlgorithm().equals(PKCSObjectIdentifiers.pbeWithSHAAnd3_KeyTripleDES_CBC))
         {
             fail("key encryption algorithm wrong");
         }
@@ -958,27 +970,27 @@ public class PKCS12StoreTest
         // check the key encryption
 
         // check the certificate encryption
-        EncryptedData cb = new EncryptedData((ASN1Sequence)c2.getContent());
+        EncryptedData cb = EncryptedData.getInstance(c2.getContent());
 
         if (type.endsWith("3DES"))
         {
-            if (!cb.getEncryptionAlgorithm().getObjectId().equals(PKCSObjectIdentifiers.pbeWithSHAAnd3_KeyTripleDES_CBC))
+            if (!cb.getEncryptionAlgorithm().getAlgorithm().equals(PKCSObjectIdentifiers.pbeWithSHAAnd3_KeyTripleDES_CBC))
             {
-                fail("expected 3DES found: " + cb.getEncryptionAlgorithm().getObjectId());
+                fail("expected 3DES found: " + cb.getEncryptionAlgorithm().getAlgorithm());
             }
         }
         else if (type.endsWith("40RC2"))
         {
-            if (!cb.getEncryptionAlgorithm().getObjectId().equals(PKCSObjectIdentifiers.pbewithSHAAnd40BitRC2_CBC))
+            if (!cb.getEncryptionAlgorithm().getAlgorithm().equals(PKCSObjectIdentifiers.pbeWithSHAAnd40BitRC2_CBC))
             {
-                fail("expected 40 bit RC2 found: " + cb.getEncryptionAlgorithm().getObjectId());
+                fail("expected 40 bit RC2 found: " + cb.getEncryptionAlgorithm().getAlgorithm());
             }
         }
         else
         {
-            if (!cb.getEncryptionAlgorithm().getObjectId().equals(PKCSObjectIdentifiers.pbewithSHAAnd40BitRC2_CBC))
+            if (!cb.getEncryptionAlgorithm().getAlgorithm().equals(PKCSObjectIdentifiers.pbeWithSHAAnd40BitRC2_CBC))
             {
-                fail("expected 40 bit RC2 found: " + cb.getEncryptionAlgorithm().getObjectId());
+                fail("expected 40 bit RC2 found: " + cb.getEncryptionAlgorithm().getAlgorithm());
             }
         }
     }
@@ -1048,7 +1060,7 @@ public class PKCS12StoreTest
         byte[] data = PKCS12Util.convertToDefiniteLength(pkcs12);
         kS.load(new ByteArrayInputStream(data), passwd);     // check MAC
 
-        DEREncodable obj = new ASN1StreamParser(data).readObject();
+        ASN1Encodable obj = new ASN1StreamParser(data).readObject();
         if (!(obj instanceof DERSequenceParser))
         {
             fail("Failed DER conversion test.");
@@ -1063,7 +1075,7 @@ public class PKCS12StoreTest
             fail("Failed deep DER conversion test - outer.");
         }
 
-        Pfx pfx = new Pfx(ASN1Sequence.getInstance(obj.getDERObject()));
+        Pfx pfx = Pfx.getInstance(obj);
 
         obj = new ASN1StreamParser(ASN1OctetString.getInstance(pfx.getAuthSafe().getContent()).getOctets()).readObject();
         if (!(obj instanceof DERSequenceParser))
