@@ -8,10 +8,15 @@ import org.bouncycastle.util.test.TestResult;
 public class RegressionTest
 {
     public static Test[]    tests = {
+        new BcPGPKeyRingTest(),
         new PGPKeyRingTest(),
+        new BcPGPRSATest(),
         new PGPRSATest(),
+        new BcPGPDSATest(),
         new PGPDSATest(),
+        new BcPGPDSAElGamalTest(),
         new PGPDSAElGamalTest(),
+        new BcPGPPBETest(),
         new PGPPBETest(),
         new PGPMarkerTest(),
         new PGPPacketTest(),
@@ -30,6 +35,10 @@ public class RegressionTest
         {
             TestResult  result = tests[i].perform();
             System.out.println(result);
+            if (result.getException() != null)
+            {
+                result.getException().printStackTrace();
+            }
         }
     }
 }
